@@ -36,9 +36,12 @@
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-tar.h"
 
+#define ACTIVITY_DELAY 20
+
 static void fr_command_tar_class_init  (FRCommandTarClass *class);
 static void fr_command_tar_init        (FRCommand         *afile);
 static void fr_command_tar_finalize    (GObject           *object);
@@ -253,9 +256,9 @@ static void
 process_line__generic (char     *line, 
 		       gpointer  data,
 		       char     *action_msg)
-{
-	FRCommand  *comm = FR_COMMAND (data);
-	char       *msg;
+{	
+	FRCommand *comm = FR_COMMAND (data);
+	char      *msg;
 
 	if (line == NULL)
 		return;
@@ -263,12 +266,14 @@ process_line__generic (char     *line,
 	if (line[strlen (line) - 1] == '/') /* ignore directories */
 		return;
 
-	msg = g_strconcat (action_msg, line, NULL);
+	comm->n_file++;
+
+	msg = g_strconcat (action_msg, file_name_from_path (line), NULL);
 	fr_command_message (comm, msg);
 	g_free (msg);
 
 	if (comm->n_files != 0) {
-		double fraction = (double) comm->n_file++ / comm->n_files;
+		double fraction = (double) comm->n_file / (comm->n_files+1);
 		fr_command_progress (comm, fraction);
 	}
 }
@@ -792,6 +797,10 @@ fr_command_tar_class_init (FRCommandTarClass *class)
 static void 
 fr_command_tar_init (FRCommand *comm)
 {
+	FRCommandTar *comm_tar = (FRCommandTar*) comm;
+
+	comm->file_type = FR_FILE_TYPE_TAR;
+
 	comm->propCanModify                = TRUE;
 	comm->propAddCanUpdate             = TRUE;
 	comm->propAddCanReplace            = FALSE; 
@@ -800,6 +809,8 @@ fr_command_tar_init (FRCommand *comm)
 	comm->propExtractCanJunkPaths      = FALSE;
 	comm->propPassword                 = FALSE;
 	comm->propTest                     = FALSE;
+
+	comm_tar->msg = NULL;
 }
 
 
@@ -818,6 +829,11 @@ fr_command_tar_finalize (GObject *object)
 		comm_tar->uncomp_filename = NULL;
 	}
 
+	if (comm_tar->msg != NULL) {
+		g_free (comm_tar->msg);
+		comm_tar->msg = NULL;
+	}
+
 	/* Chain up */
         if (G_OBJECT_CLASS (parent_class)->finalize)
 		G_OBJECT_CLASS (parent_class)->finalize (object);
